

;												
; SPI-MASTER fr ATmega8						
; Bernhard.Erfurt@gmx.de						
; 03/2006										
;												
; Nach einer Sekunde wird 						
; ein ZEICHEN zum SLAVE gesendet und 			
; ein ZEICHEN vom SLAVE empfangen und das 		
; empfangene ZEICHEN vom SLAVE an PORTD (8xLED) 
; angezeigt										
; 												


; ATmega8				
.include "m8def.inc"

; ERROR-LED an PORTB PIN 0	
.equ B_LED  = 0

; REGISTER definieren	
.def SENDE_DATA	= R1
.def temp 		= r16
.def temp1 		= r17
.def temp2 		= r18
.def temp3 		= r19

.cseg			;Beginn eines Code-Segmentes			  
.org 0			;Startadresse=0							  

; Interruptvectoren fr ATmega 8 !!!					  
rjmp RESET      ; Reset Handler                           


RESET:                        
;STACK initialisieren
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp

; PORT B initialisieren			
	ldi temp, 0b00101101	; AUSGANG: SCK MOSI SS LED	
	out DDRB, temp
; PORT B setzen					
	out PORTB, temp			; ALLE AUSGNGE auf HIGH	
; PORT C initialisieren			
	ldi temp, 0b00000000	; alles EINGNGE			
	out DDRC, temp
; PORT D initialisieren			
	ldi temp, 0b11111111	; alles Ausgnge fr 8x LED
	out DDRD, temp
; PORT D setzen					
	out PORTD, temp			; ALLE AUSGNGE auf HIGH	

; SPI_INITIALISIERUNG
	rcall SPI_INITIALISIERUNG

; STARTWERTE
	clr SENDE_DATA


; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	rcall wait_1s			; 1s PAUSE					
	
	inc SENDE_DATA			; +1						
	mov temp, SENDE_DATA	; KOPIEREN nach TEMP		
	com temp				; INVERTIEREN				
	rcall SPI_SENDEN		; SPI SENDEN 	INP: TEMP	
	rcall SPI_SPSR_AUSWERTEN; SPSR-Rgister auswerten	

	rcall SPI_EMPFANGEN		; SPI EMPFANGEN OUT: TEMP	
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; LED leuchtet, wenn 															
; WCOL(Write COLlision Flag) gesetzt ist										
; SPSR => SPIF WCOL - - - - - SPI2X  (SPI Status Register )					
SPI_SPSR_AUSWERTEN:
	in temp, SPSR			; einlesen	
	andi temp, 0b01000000 	; BITMUSTER	
	tst temp
	brne SPI_SPSR_AUSWERTEN_ERROR
	sbi PORTB, B_LED		; LED aus	
	ret
SPI_SPSR_AUSWERTEN_ERROR:
	cbi PORTB, B_LED		; LED an	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_INITIALISIERUNG:
; SPCR => SPIE SPE DORD MSTR CPOL CPHA SPR1 SPR0 (SPI Control Register)		
; 																			
; SPIE: SPI Interrupt Enable												
; SPE:  SPI Enable															
; DORD: Data Order															
; MSTR: Master/Slave Select													
; CPOL: Clock Polarity														
; CPHA: Clock Phase															
;						   Leading Edge 	Trailing Edge 		SPI Mode	
;		CPOL = 0, CPHA = 0 Sample (Rising) 	Setup (Falling) 		0		
;		CPOL = 0, CPHA = 1 Setup (Rising) 	Sample (Falling) 		1		
;		CPOL = 1, CPHA = 0 Sample (Falling) Setup (Rising) 			2		
;		CPOL = 1, CPHA = 1 Setup (Falling) 	Sample (Rising) 		3		
; SPR1: SPI Clock Rate Select 1												
; SPR0: SPI Clock Rate Select 0												
; 		SPR1 SPR0 SCK Frequency												
;  		0    0   fosc/4														
;  		0    1   fosc/16													
;  		1    0   fosc/64													
;  		1    1   fosc/128													
;																			
; Enable SPI, Master, set clock rate fck/128								
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(1<<SPR1)|(1<<SPR0)
	out SPCR,temp

; SPSR => SPIF WCOL - - - - - SPI2X  (SPI Status Register )					
;																			
; SPIF:  SPI Interrupt Flag													
; WCOL:  Write COLlision Flag												
; SPI2X: Double SPI Speed Bit												
;																			
; Double SPI Speed = off													
	ldi temp,(0<<SPI2X)
	out SPSR,temp	

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; input: TEMP																	
;																				
;	rcall SPI_SENDEN	; INP: TEMP												
;																				
SPI_SENDEN:
; MASTER SS auf LOW				
	cbi PORTB, 2
; Start transmission of data 	
	out SPDR,temp
; Wait for transmission complete
SPI_SENDEN_w:
	sbis SPSR,SPIF
	rjmp SPI_SENDEN_w
; MASTER SS auf HIGH				
	sbi PORTB, 2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_EMPFANGEN:
	in temp, SPDR	; einlesen			

	out PORTD, temp ; auf PORT D ausgeben

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; bei 1 MHz Takt ca.1s Pause													
WAIT_1s:
	push temp1           ; temp1 auf dem Stack sichern            
	push temp2           ; temp2 auf dem Stack sichern            
	push temp3
;------------
	ldi temp3,6
	clr temp2
	clr	temp1			
wait1s_w:	
	dec	temp1
	brne	wait1s_w
	dec	temp2		
	brne	wait1s_w
	dec	temp3		
	brne	wait1s_w
;------------
	pop temp3
	pop temp2      		; temp2 wiederherstellen                    
	pop temp1      		; temp1 wiederherstellen                    
ret
