

;												
; SPI-SLAVE fr ATmega8							
; Bernhard.Erfurt@gmx.de						
; 03/2006										
;												
; Das vom MASTER gesendete Zeichen wird 		
; an PORT-D ausgegeben und wieder zurckgesendet
;												


; ATmega8			
.include "m8def.inc"



; ERROR-LED an PORTB PIN 0
.equ B_LED  = 0

; REGISTER			

.def temp 	= r16
.def temp1 	= r17
.def temp2 	= r18
.def temp3 	= r19
.def temp4 	= r20
.def temp5 	= r21
.def temp6 	= r22
.def temp7 	= r23
.def temp8 	= r24
.def temp9 	= r25



.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0				

rjmp RESET      ; Reset Handler                           



RESET:                        
;STACK initialisieren
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp



; PORT B initialisieren			
	ldi temp, 0b00010001	; AUSGANG:  MISO  LED		
	out DDRB, temp
; PORT B setzen					
	out PORTB, temp			; alle AUSGNGE auf HIGH	

; PORT C initialisieren			
	ldi temp, 0b00000000	; alles eingnge			
	out DDRC, temp
; PORT D initialisieren			
	ldi temp, 0b11111111	; alles Ausgnge fr 8x LED
	out DDRD, temp
; PORT D setzen					
	out PORTD, temp			; alle AUSGNGE auf HIGH	

; SPI_INITIALISIERUNG
rcall SPI_INITIALISIERUNG

; interrupts freigeben
	sei



; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	
	rcall SPI_EMPFANGEN			; SPI EMPFANGEN OUT: TEMP	

	rcall SPI_SPSR_AUSWERTEN	; REGISTER SPSR auswerten	


rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; LED leuchtet, wenn 															
; WCOL(Write COLlision Flag) gesetzt ist										
; SPSR => SPIF WCOL - - - - - SPI2X  (SPI Status Register )					
SPI_SPSR_AUSWERTEN:
	in temp, SPSR			; einlesen	
	andi temp, 0b01000000 	; BIT herrausfiltern
	tst temp				; ist 0 ?	
	brne SPI_SPSR_AUSWERTEN_ERROR
	sbi PORTB, B_LED		; LED aus	
	ret
SPI_SPSR_AUSWERTEN_ERROR:
	cbi PORTB, B_LED		; LED an	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_INITIALISIERUNG:
; SPCR => SPIE SPE DORD MSTR CPOL CPHA SPR1 SPR0 (SPI Control Register)		
; 																			
; SPIE: SPI Interrupt Enable												
; SPE:  SPI Enable															
; DORD: Data Order															
; MSTR: Master/Slave Select													
; CPOL: Clock Polarity														
; CPHA: Clock Phase															
;						   Leading Edge 	Trailing Edge 		SPI Mode	
;		CPOL = 0, CPHA = 0 Sample (Rising) 	Setup (Falling) 		0		
;		CPOL = 0, CPHA = 1 Setup (Rising) 	Sample (Falling) 		1		
;		CPOL = 1, CPHA = 0 Sample (Falling) Setup (Rising) 			2		
;		CPOL = 1, CPHA = 1 Setup (Falling) 	Sample (Rising) 		3		
; SPR1: SPI Clock Rate Select 1												
; SPR0: SPI Clock Rate Select 0												
; 		SPR1 SPR0 SCK Frequency												
;  		0    0   fosc/4														
;  		0    1   fosc/16													
;  		1    0   fosc/64													
;  		1    1   fosc/128													


; Enable SPI
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(0<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out SPCR,temp

; SPSR => SPIF WCOL - - - - - SPI2X  (SPI Status Register )					
;																			
; SPIF:  SPI Interrupt Flag													
; WCOL:  Write COLlision Flag												
; SPI2X: Double SPI Speed Bit												

; Double SPI Speed = off					
	ldi temp,(0<<SPI2X)
	out SPSR,temp	

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_EMPFANGEN:

; Wait for reception complete
	sbis SPSR,SPIF
	rjmp SPI_EMPFANGEN
; Read received data and return
	in temp,SPDR	; einlesen				
	out PORTD, temp ; auf PORT D ausgeben	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

