

;													
; SPI-SLAVE fr ATmega8								
; Bernhard.Erfurt@gmx.de							
; 03/2006											
;													
; Wird ein Zeichen ber SPI vom MASTER empfangen, 	
; dann wird ein Interrupt ausgelst und das 		
; empfangene Zeichen am Port-D ausgegeben (8xLED)	
;													

; ATmega8			
.include "m8def.inc"

; LED an PORTB PIN 0
.equ B_LED  = 0

; REGISTER			
.def EMPFANGS_DATEN	= R1

.def temp 			= R16

.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

; Interrupt-Vectoren fr ATmega8 !!!  					  
rjmp RESET      ; Reset Handler                           
reti ;rjmp EXT_INT0   ; IRQ0 Handler                      
reti ;rjmp EXT_INT1   ; IRQ1 Handler                      
reti ;rjmp TIM2_COMP  ; Timer2 Compare Handler            
reti ;rjmp TIM2_OVF   ; Timer2 Overflow Handler           
reti ;rjmp TIM1_CAPT  ; Timer1 Capture Handler            
reti ;rjmp TIM1_COMPA ; Timer1 CompareA Handler           
reti ;rjmp TIM1_COMPB ; Timer1 CompareB Handler           
reti ;rjmp TIM1_OVF   ; Timer1 Overflow Handler           
reti ;rjmp TIM0_OVF   ; Timer0 Overflow Handler           
rjmp SPI_INTERRUPT    ; SPI Transfer Complete Handler     
reti ;rjmp USART_RXC  ; USART RX Complete Handler         
reti ;rjmp USART_UDRE ; UDR Empty Handler                 
reti ;rjmp USART_TXC  ; USART TX Complete Handler         
reti ;rjmp ADC        ; ADC Conversion Complete Handler   
reti ;rjmp EE_RDY     ; EEPROM Ready Handler              
reti ;rjmp ANA_COMP   ; Analog Comparator Handler         
reti ;rjmp TWSI       ; Two-wire Serial Interface Handler 
reti ;rjmp SPM_RDY    ; Store Program Memory Ready Handler


RESET:                        
;STACK initialisieren
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; PORT B initialisieren			
	ldi temp, 0b00010001	; AUSGANG:  MISO  LED		
	out DDRB, temp
; PORT B setzen					
	out PORTB, temp			; alle Ausgnge auf HIGH	

; PORT C initialisieren			
	ldi temp, 0b00000000	; alles Eingnge			
	out DDRC, temp
; PORT D initialisieren			
	ldi temp, 0b11111111	; alles Ausgnge fr 8x LED
	out DDRD, temp
; PORT D setzen					
	out PORTD, temp			; alle Ausgnge auf HIGH	

; SPI_INITIALISIERUNG
	rcall SPI_INITIALISIERUNG

; STARTWERTE
	clr EMPFANGS_DATEN

; interrupts freigeben
	sei



; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	



rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
;Dieser interupt wird aufgerufen, wenn ein Zeichen vom Master Empfangen wurde	
;																				
; OUT: EMPFANGS_DATEN															
;																				
SPI_INTERRUPT:
push TEMP		    ; Sichern auf Stack						
in	 TEMP,SREG		; Einlesen des SREG 					
push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)	

;-----------------

	rcall SPI_EMPFANGEN			; SPI EMPFANGEN OUT: TEMP	

	rcall SPI_SPSR_AUSWERTEN	; Fehlerprfung				

;-----------------
pop	TEMP			; LESEN von SREG vom STACK (KOPIE)		
out	SREG,TEMP		; Wiederherstellen von SREG       		
pop	TEMP			; Wiederherstellen von "TEMP"     		

reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
; LED leuchtet, wenn 															
; WCOL(Write COLlision Flag) gesetzt ist										
; SPSR => SPIF WCOL - - - - - SPI2X  (SPI Status Register )					
SPI_SPSR_AUSWERTEN:
	in temp, SPSR			; einlesen
	andi temp, 0b01000000 	; BIT herrausfiltern
	tst temp
	brne SPI_SPSR_AUSWERTEN_ERROR
	sbi PORTB, B_LED		; LED aus	
	ret
SPI_SPSR_AUSWERTEN_ERROR:
	cbi PORTB, B_LED		; LED an	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_INITIALISIERUNG:
; SPCR => SPIE SPE DORD MSTR CPOL CPHA SPR1 SPR0 (SPI Control Register)		
; 																			
; SPIE: SPI Interrupt Enable												
; SPE:  SPI Enable															
; DORD: Data Order															
; MSTR: Master/Slave Select													
; CPOL: Clock Polarity														
; CPHA: Clock Phase															
;						   Leading Edge 	Trailing Edge 		SPI Mode	
;		CPOL = 0, CPHA = 0 Sample (Rising) 	Setup (Falling) 		0		
;		CPOL = 0, CPHA = 1 Setup (Rising) 	Sample (Falling) 		1		
;		CPOL = 1, CPHA = 0 Sample (Falling) Setup (Rising) 			2		
;		CPOL = 1, CPHA = 1 Setup (Falling) 	Sample (Rising) 		3		
; SPR1: SPI Clock Rate Select 1												
; SPR0: SPI Clock Rate Select 0												
; 		SPR1 SPR0 SCK Frequency												
;  		0    0   fosc/4														
;  		0    1   fosc/16													
;  		1    0   fosc/64													
;  		1    1   fosc/128													
; 																			
; SPI Interrupt Enable,Enable SPI											
	ldi temp,(1<<SPIE)|(1<<SPE)|(0<<DORD)|(0<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out SPCR,temp

; SPSR => SPIF WCOL - - - - - SPI2X  (SPI Status Register )					
;																			
; SPIF:  SPI Interrupt Flag													
; WCOL:  Write COLlision Flag												
; SPI2X: Double SPI Speed Bit												
	ldi temp,(0<<SPI2X)
	out SPSR,temp	

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; OUT: TEMP	+ EMPFANGS_DATEN													
;																				
SPI_EMPFANGEN:
	in temp,SPDR				; SPI-DATEN-REGISTER einlesen	
	mov EMPFANGS_DATEN, temp	; KOPIE nach EMPFANGS_DATEN
	out PORTD, EMPFANGS_DATEN 	; auf PORT D ausgeben			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################


